#include "p2p_wrapper.h"
#include "gio/gnetworking.h"
#include "json/json.h"
#include "p2p_utilities.h"

#pragma comment(lib, "ffi.lib")
#pragma comment(lib, "gio-2.0.lib")
#pragma comment(lib, "glib-2.0.lib")
#pragma comment(lib, "gmodule-2.0.lib")
#pragma comment(lib, "gobject-2.0.lib")
#pragma comment(lib, "gthread-2.0.lib")
#pragma comment(lib, "intl.lib")
#pragma comment(lib, "libcrypto.lib")
#pragma comment(lib, "libssl.lib")
#pragma comment(lib, "z.lib")
#pragma comment(lib, "nice.lib")

namespace offcn
{
	std::thread  *P2pWrapper::net_thread_ = NULL;
	GMainLoop	*P2pWrapper::g_main_loop_ = NULL;

	bool P2pWrapper::InitP2pEnv()
	{
		g_networking_init();
		g_main_loop_ = g_main_loop_new(NULL, false);
		if (g_main_loop_ == NULL)
		{
			return false;
		}

		net_thread_ = new std::thread(NetThreadCallback);

		return true;
	}
	void P2pWrapper::UnInitP2pEnv()
	{
		if (g_main_loop_)
		{
			g_main_loop_quit(g_main_loop_);
			g_main_loop_unref(g_main_loop_);
		}

		if (net_thread_)
		{
			net_thread_->join();
			delete net_thread_;
			net_thread_ = NULL;
		}
	}
	P2pWrapper::P2pWrapper()
		:agent_(NULL),
		stream_id_(0)
	{
	}
	P2pWrapper::~P2pWrapper()
	{
		Stop();
	}
	bool P2pWrapper::Start(std::string stun_ip, int stun_port, bool is_controller)
	{
		GMainContext *ctx = g_main_loop_get_context(g_main_loop_);
		agent_ = nice_agent_new(ctx, NICE_COMPATIBILITY_RFC5245);
		if (agent_ == NULL)
		{
			return false;
		}

		g_object_set(agent_, "stun-server", stun_ip.c_str(), NULL);
		g_object_set(agent_, "stun-server-port", stun_port, NULL);
		if (is_controller)
		{
			g_object_set(agent_, "controlling-mode", 1, NULL);
		}
		else
		{
			g_object_set(agent_, "controlling-mode", 0, NULL);
		}

		g_signal_connect(agent_, "candidate-gathering-done", G_CALLBACK(cb_candidate_gather_done), NULL);
		g_signal_connect(agent_, "new-selected-pair", G_CALLBACK(cb_new_selected_pair), NULL);
		g_signal_connect(agent_, "component-state-changed", G_CALLBACK(cb_component_state_changed), NULL);

		stream_id_ = nice_agent_add_stream(agent_, 1);
		if (stream_id_ == 0)
		{
			goto Error;
		}

		// Attach to the component to receive the data
		// Without this call, candidates cannot be gathered
		nice_agent_attach_recv(agent_, stream_id_, 1, ctx, cb_nice_recv, NULL);

		// Start gathering local candidates
		if (!nice_agent_gather_candidates(agent_, stream_id_))
		{
			goto Error;
		}

		return true;

	Error:
		Stop();
		return false;
	}
	void P2pWrapper::Stop()
	{
		if (agent_)
		{
			g_object_unref(agent_);
		}
	}

	bool P2pWrapper::SetRemoteCandidate(std::string candidates)
	{
		GSList *candidate_list = NULL;

		std::string ufrag;
		std::string password;

		if (!ParseCandidatesString(candidates, stream_id_, ufrag, password, &candidate_list)) return false;

		if (!nice_agent_set_remote_credentials(agent_, stream_id_, ufrag.c_str(), password.c_str()))
		{
			goto Error;
		}
		if(!nice_agent_set_remote_candidates(agent_, stream_id_, 1, candidate_list))
		{
			goto Error;
		}

		return true;

	Error:
		if (candidate_list)
		{
			g_slist_free_full(candidate_list, (GDestroyNotify)&nice_candidate_free);
		}

		return false;
	}
	int  P2pWrapper::SendData(const char *buffer, int len)
	{
		return nice_agent_send(agent_, stream_id_, 1, len, buffer);
	}
	void P2pWrapper::NetThreadCallback()
	{
		g_main_loop_run(g_main_loop_);
	}

	int  P2pWrapper::candidate_gather_done(NiceAgent *agent, guint stream_id)
	{
		std::string candidates;
		if (BuildCandidatesString(agent, stream_id, candidates))
		{
			printf("--->local candidates = %s\n", candidates.c_str());
			FILE *file = fopen("./1.txt", "ab+");
			fwrite(candidates.c_str(), 1, candidates.length(), file);
			fclose(file);

			return 0;
		}

		return -1;
	}
	void P2pWrapper::new_selected_pair(NiceAgent *agent, guint stream_id, guint component_id, gchar *lfoundation, gchar *rfoundation)
	{
		printf("----new_selected_pair----\n");
	}
	void P2pWrapper::component_state_changed(NiceAgent *agent, guint stream_id, guint component_id, guint state)
	{
		printf("----component_state_changed----%u\n", state);
	}
	void P2pWrapper::nice_recv(NiceAgent *agent, guint stream_id, guint component_id, guint len, gchar *buf)
	{
		char szBuffer[1500] = { 0 };
		memcpy(szBuffer, buf, len);
		printf("----nice_recv----%s, len = %d\n", szBuffer, len);
	}
}